/////////////////////////////////////////////////////////////////////
// ****FUNCTION: simuser**** PARA PSIM 9.0.3 32 bit
// ****FUNCTION: SimulationStep**** PARA PSIM 2022 64 bit
//double t: (read only) time
//double delt: (read only) time step as in Simulation control
//double *in: (read only) zero based array of input values. in[0] is the first node, in[1] second input...
//double *out: (write only) zero based array of output values. out[0] is the first node, out[1] second output...
//int *pnError: (write only)  assign  *pnError = 1;  if there is an error and set the error message in szErrorMsg
//    strcpy(szErrorMsg, "Error message here..."); 
// DO NOT CHANGE THE NAME OR PARAMETERS OF THIS FUNCTION

// Las entradas del sistema son
// in[0]= ref;      referencia
// in[1]= sal;      variable a controlar
// in[2]= Fcpu;     frecuencia del oscilador de la uC
// in[3]= 0;
// in[4]= 0;
// in[5]= 0;
// in[6]= 0;

// Incluye librerias especiales o datos .h
//#include "dll.h"
//#include <windows.h>
//#include "ganancias.h"
#define		Vpp		1			// Ganacia del actuador 

// Coeficientes del controlador necesario:
// Ec. recursiva: u[k] = K*ev[k] + aK*ev[k-1] + u[k-1];
#define		K		 0.1721    //1.011
#define		aK		-0.1158     //-0.664
// Registros internos del microcontrolador
#define		PER1	 75	   // Define el periodo del Timer1 (0.0005s). 0.000362
#define		PRESC1	 4		   // Define el prescaler del Timer1.

// Para flags y dems:
#define		OFF		0			// Apagado.
#define		ON		1			// Encendido.

// PER1: Para timer de 16 bits, puede ser cualquier valor entero de 1 a 65535.
// PRESC1: Puede ser 1, 2, 4, 16, 32.
// Periodo de muestreo: Ts = PER1*PRESC1*Tosc
//
// Ejemplo: fs=2000 Hz; Ts = 0.0005 seg, Fosc = 500kHz y PRESC1 = 32 => PER1 = Ts*Fosc/PRESC1 = 156,25.
// Se implementa un contador ascendente y descendente

// PROGRAMA QUE SE EJECUTA EN CADA PASO DE SIMULACION DEL PSIM osea a 1us
//****Funcin para PSIM 9.0.3**** Compilar usando TDM-GCC 32 bit
//void __declspec(dllexport) simuser (double t, double delt, double *in, double *out){

//****Funcin para PSIM 2022**** Compilar usando TDM-GCC 64 bit
void SimulationStep(double t, double delt, double *in, double *out){
  	// variables reales
	static double y=0;   					 // variable controlada
   	static double uk=0, ukm1=0, uc=0, uk1=0; // Accin de control
   	static double evk=0, evkm1=0;	// Error actual y anterior
   	static double ref;				// Referencia
	static double Ts;    			/* Periodo de muestreo */
	static double Fcpu, Tcpu; 		// Variables p/calculo de la relacin Ncont.
	
	// Variables enteras
	static int contador=0, Ncont, direccion = 1;
	static int PWM1, PWM2;
	static int ComparReg1, ComparReg2;
	
	// Variables booleanas:
	static char FLAG_calcu;				// Flag p/indicar si se calcula la acc. de control.
	static char FLAG_muestreo=OFF;		// Flag p/indicar si se muestrea.
	
	// Se calcula el numero de conteos en un periodo: Valor mximo del contador
	//----------------------------------------------------------------------------------
	// CLCULOS INICIALES
	//----------------------------------------------------------------------------------
   	Fcpu = in[2];					// Adquiere frecuencia del oscilador					
	Tcpu = 1/Fcpu;					// y calcula periodo de la CPU.
	Ts = (PER1*PRESC1)/Fcpu;		// Calcula el periodo de muestreo
    Ncont=Ts/delt;                  // Calculo de Ncont
	
	// Se realiza el muestreo de las variables controladas
    if(FLAG_muestreo==ON){
        FLAG_muestreo = OFF;
        y = in[1]/4;
        FLAG_calcu = ON;
    }
	
	// Calculo de la estrategia de control
	if(FLAG_calcu==ON){
		FLAG_calcu = OFF;
		ref = in[0]; 					// Adquiere la referencia (igual que generar adentro del uC)
		/*********************************************************************************/
		/*  Se calcula el error entre la salida de la planta y la referencia             */
		/*********************************************************************************/
		evk = ref - y;
		/*********************************************************************************/
		/*                   Calculo de la estrategia de control                         */
		/*********************************************************************************/
		uk = K*evk + aK*evkm1 + ukm1;			// Calcula acc. de control del PI
		evkm1 = evk;							// Error de tensin anterior.
		ukm1 = uk;								// Accion de control anterior del PI
		//Para lazo abierto
		//uk = ref; 
		uc = uk*Ncont;			// Accin de control comparable con PWM
	} //termina el calculo de la estrategia de control

    //**********************************************************//
	// Incrementamos o decrementamos el contador segun corresponda
	if(contador == 0){
    	FLAG_muestreo = ON;
	}
	contador = contador + 1;
	if(contador == Ncont){
		contador = 0;
		ComparReg1 = uc;
		uk1 = uk; // debido al atraso de implementacin digital se 
		//actualiza en este momento la salida de la accin de control
	}

	// Maquina de estado que define el patron PWM
    if(ComparReg1 > contador)
        PWM1 = 1;
    else
        PWM1 = 0;
    
	// Se actualizan los valores de las salidas
	out[0] = PWM1; //brazo 1
	//out[1] = PWM2; //brazo 2
	out[2] = uk1;
	out[3] = ref;
	out[4] = y;
	out[5] = evk;
	out[6] = contador;
	out[7] = ComparReg1;
}
